#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "database.h"
#include "proxy.h"
#include "config.h"

#define PACKAGE_NAME "WICTP"
#define DEFAULT_DATABASE "data.sqlite"
#define DEFAULT_CONFIG "config.json"
#define DEFAULT_PORT 8080

unsigned int validatePortNumber(unsigned int portNumber){
	// Given that we are dealing with unsined ints, we don't need to worry about
	// checking for values less than 0
	if ( portNumber > 65535 ){
		printf( "Port %i is invalid, using default of %i", portNumber, DEFAULT_PORT );
		return DEFAULT_PORT;
	}
	return portNumber;
}

void printHelp(){
	printf("Usage: %s [options]\n", PACKAGE_NAME);
	printf( "Options are:\n" );
	printf( "\t --config OPTION VALUE      Sets config option OPTION to VALUE\n" );
	printf( "\t --print-config             Prints current config\n" );
}

int main(int argc, char**argv){
	Config *config = configDefaults();


	for ( unsigned int i = 1; i < argc; i++ ){
		if ( strcmp( argv[i], "--config" ) == 0 ){
			if ( i + 2 < argc ){
				setConfig( config, argv[i + 1], argv[i + 2] );
				i += 2;
			} else {
				printf("--config requires 2 positional arguments\n");
			}
		} else if ( strcmp( argv[i], "--print-config" ) == 0 ){
			printConfig(config);
			return 0;
		} else if ( strcmp( argv[i], "--help" ) == 0 ){
			printHelp();
			return 0;
		} else {
			printf("Unknown option %s\n", argv[i]);
		}
	}

	if (path_exists( config->userConfig ) ){
		printf("User config file exists but still needs to be parsed\n");
	}

	if (path_exists( config->localConfig ) ){
		printf("Local config file exists but still needs to be parsed\n");
	}

	if ( !db_file_exists(config->database) ){
		printf("Creating DB");
		db_create(config->database);
	}

	proxy_startListener(config->port);

	return 0;
}
